@@ -4,11 +4,11 @@ The main entry point for RPC4Django. Usually, the user simply puts
 
 ::
 
-    urlpatterns = patterns('', 
-        # rpc4django will need to be in your Python path  
-        (r'^RPC2$', 'rpc4django.views.serve_rpc_request'), 
+    urlpatterns = patterns('',
+        # rpc4django will need to be in your Python path
+        (r'^RPC2$', 'rpc4django.views.serve_rpc_request'),
     )
-    
+
 '''
 
 import logging
@@ -34,13 +34,13 @@ RESTRICT_OOTB_AUTH = getattr(settings,
                              'RPC4DJANGO_RESTRICT_OOTB_AUTH', True)
 RESTRICT_JSON = getattr(settings, 'RPC4DJANGO_RESTRICT_JSONRPC', False)
 RESTRICT_XML = getattr(settings, 'RPC4DJANGO_RESTRICT_XMLRPC', False)
-RESTRICT_METHOD_SUMMARY = getattr(settings, 
+RESTRICT_METHOD_SUMMARY = getattr(settings,
                                   'RPC4DJANGO_RESTRICT_METHOD_SUMMARY', False)
 RESTRICT_RPCTEST = getattr(settings, 'RPC4DJANGO_RESTRICT_RPCTEST', False)
 RESTRICT_RPCTEST = getattr(settings, 'RPC4DJANGO_RESTRICT_RPCTEST', False)
-HTTP_ACCESS_CREDENTIALS = getattr(settings, 
+HTTP_ACCESS_CREDENTIALS = getattr(settings,
                                   'RPC4DJANGO_HTTP_ACCESS_CREDENTIALS', False)
-HTTP_ACCESS_ALLOW_ORIGIN = getattr(settings, 
+HTTP_ACCESS_ALLOW_ORIGIN = getattr(settings,
                                   'RPC4DJANGO_HTTP_ACCESS_ALLOW_ORIGIN', '')
 
 # get a list of the installed django applications
@@ -51,21 +51,21 @@ def check_request_permission(request, request_format='xml'):
     '''
     Checks whether this user has permission to call a particular method
     This method does not check method call validity. That is done later
-    
+
     **Parameters**
-    
+
     - ``request`` - a django HttpRequest object
-    - ``request_format`` - the request type: 'json' or 'xml' 
-    
+    - ``request_format`` - the request type: 'json' or 'xml'
+
     Returns ``False`` if permission is denied and ``True`` otherwise
     '''
-    
+
     user = getattr(request, 'user', None)
     methods = dispatcher.list_methods()
     method_name = dispatcher.get_method_name(request.raw_post_data, \
                                              request_format)
     response = True
-    
+
     for method in methods:
         if method.name == method_name:
             # this is the method the user is calling
@@ -83,39 +83,51 @@ def check_request_permission(request, request_format='xml'):
                     response = False
                 else:
                     logger.debug('User "%s" is authorized' %(str(user)))
+            elif method.login_required:
+                logger.debug('Method "%s" is protected by login_required'% method.name)
+                if user is None:
+                    # user is only none if not using AuthenticationMiddleware
+                    logger.warn('AuthenticationMiddleware is not enabled')
+                    response = False
+                elif user.is_anonymous():
+                    # ensure the user is logged in
+                    logger.info('User "%s" is NOT authorized' %(str(user)))
+                    response = False
+                else:
+                    logger.debug('User "%s" is authorized' %(str(user)))
             else:
                 logger.debug('Method "%s" is unprotected' %(method.name))
-                
+
             break
-    
+
     return response
-    
+
 def is_xmlrpc_request(request):
     '''
     Determines whether this request should be served by XMLRPC or JSONRPC
-    
+
     Returns ``True`` if this is an XML request and false for JSON
-    
+
     1. If there is no post data, display documentation
     2. content-type = text/xml or application/xml => XMLRPC
     3. content-type contains json or javascript => JSONRPC
     4. Try to parse as xml. Successful parse => XMLRPC
     5. JSONRPC
-    
+
     '''
-    
+
     conttype = request.META.get('CONTENT_TYPE', 'unknown type')
-    
+
     # check content type for obvious clues
     if conttype == 'text/xml' or conttype == 'application/xml':
         return True
     elif conttype.find('json') >= 0 or conttype.find('javascript') >= 0:
         return False
-    
+
     if LOG_REQUESTS_RESPONSES:
         logger.info('Unrecognized content-type "%s"' %conttype)
         logger.info('Analyzing rpc request data to get content type')
-    
+
     # analyze post data to see whether it is xml or json
     # this is slower than if the content-type was set properly
     try:
@@ -123,7 +135,7 @@ def is_xmlrpc_request(request):
         return True
     except ExpatError:
         pass
-    
+
     return False
 
 def serve_rpc_request(request):
@@ -131,81 +143,81 @@ def serve_rpc_request(request):
     Handles rpc calls based on the content type of the request or
     returns the method documentation page if the request
     was a GET.
-    
+
     **Parameters**
-    
+
     ``request``
         the Django HttpRequest object
-        
+
     '''
 
     if request.method == "POST" and len(request.POST) > 0:
         # Handle POST request with RPC payload
-        
+
         if LOG_REQUESTS_RESPONSES:
             logger.debug('Incoming request: %s' %str(request.raw_post_data))
-            
+
         if is_xmlrpc_request(request):
             if RESTRICT_XML:
                 raise Http404
-            
+
             if not check_request_permission(request, 'xml'):
                 return HttpResponseForbidden()
-            
+
             resp = dispatcher.xmldispatch(request.raw_post_data, \
                                           request=request)
             response_type = 'text/xml'
         else:
             if RESTRICT_JSON:
                 raise Http404
-            
+
             if not check_request_permission(request, 'json'):
                 return HttpResponseForbidden()
-            
+
             resp = dispatcher.jsondispatch(request.raw_post_data, \
                                            request=request)
             response_type = 'application/json'
-            
+
         if LOG_REQUESTS_RESPONSES:
             logger.debug('Outgoing %s response: %s' %(response_type, resp))
-        
+
         return HttpResponse(resp, response_type)
     elif request.method == 'OPTIONS':
         # Handle OPTIONS request for "preflighted" requests
         # see https://developer.mozilla.org/en/HTTP_access_control
-        
+
         response = HttpResponse('', 'text/plain')
-        
+
         origin = request.META.get('HTTP_ORIGIN', 'unknown origin')
         response['Access-Control-Allow-Methods'] = 'POST, GET, OPTIONS'
         response['Access-Control-Max-Age'] = 0
         response['Access-Control-Allow-Credentials'] = \
                         str(HTTP_ACCESS_CREDENTIALS).lower()
         response['Access-Control-Allow-Origin']= HTTP_ACCESS_ALLOW_ORIGIN
-        
+
         response['Access-Control-Allow-Headers'] = \
                     request.META.get('HTTP_ACCESS_CONTROL_REQUEST_HEADERS', '')
-                    
+
         if LOG_REQUESTS_RESPONSES:
             logger.debug('Outgoing HTTP access response to: %s' %(origin))
-                    
+
         return response
     else:
         # Handle GET request
-        
+
         if RESTRICT_METHOD_SUMMARY:
             # hide the documentation by raising 404
             raise Http404
-        
+
         # show documentation
         methods = dispatcher.list_methods()
         template_data = {
             'methods': methods,
             'url': URL,
-            
+
             # rpc4django version
             'version': version(),
-            
+
             # restricts the ability to test the rpc server from the docs
             'restrict_rpctest': RESTRICT_RPCTEST,
         }
@@ -232,8 +244,8 @@ try:
     URL = reverse(serve_rpc_request)
 except NoReverseMatch:
     URL = ''
-    
+
 # instantiate the rpcdispatcher -- this examines the INSTALLED_APPS
 # for any @rpcmethod decorators and adds them to the callable methods
-dispatcher = RPCDispatcher(URL, APPS, RESTRICT_INTROSPECTION, RESTRICT_OOTB_AUTH) 
+dispatcher = RPCDispatcher(URL, APPS, RESTRICT_INTROSPECTION, RESTRICT_OOTB_AUTH)
 
